const fetch = require('node-fetch');

export const getiCalFile = async (user: string, file: string, showBlockers: boolean, electiveModule: string, profileModule: string): Promise<string> => {
    try {
        let iCalFileContents = '';

        let baseUrl = 'https://rapla.dhbw-karlsruhe.de/rapla?page=ical';
        baseUrl += '&user=' + user;
        baseUrl += '&file=' + file;

        const response = await fetch(baseUrl);

        let body = await response.text();

        // Parse the ical file
        let iCalFile = parseIcal(body);

        // Now remove unwanted shit from the iCal file
        if (!showBlockers) {
            iCalFile = removeBlockers(iCalFile);
        }

        // Turn into one file again
        let resultingFile = serializeIcal(iCalFile);

        return resultingFile;
    } catch (err) {
        throw err;
    }
};

export const parseIcal = function (icalContents: string): iCalFile {
    const fileHeaderRegex: RegExp = /^BEGIN:VCALENDAR.*?BEGIN:VEVENT$/ms;
    const eventRegex: RegExp = /^BEGIN:VEVENT.*?END:VEVENT$/msg;

    let headerMatch = fileHeaderRegex.exec(icalContents);
    let header = '';

    if (headerMatch) {
        header = headerMatch[0];
    }

    header = header.substr(0, header.lastIndexOf('\n'));

    let events: string[] = [];
    let match: any;
    while (match = eventRegex.exec(icalContents)) {
        if (match) {
            events.push(match[0]);
        }
    }

    return {
        head: header,
        body: events
    };
};

export const serializeIcal = function (ical: iCalFile): string {
    let resString = ical.head;
    ical.body.forEach((event) => {
        resString += event + '\n';
    });
    resString += 'END:VCALENDAR';

    return resString;
};

export const removeBlockers = function (ical: iCalFile): iCalFile {
    let remainingEvents: string[] = [];

    ical.body.forEach((event) => {
        if (!event.includes('CATEGORIES:Sonstiger Termin') && !event.includes('SUMMARY:Präsenz')) {
            remainingEvents.push(event);
        }
    });

    ical.body = remainingEvents;

    return ical;
};

export interface iCalFile {
    head: string;
    body: string[];
}
